@@ -1,57 +1,67 @@
 //! A simple IRC crate written in rust
 //! ```no_run
-//! use circe::{commands::Command, Client, Config};
-//! fn main() -> Result<(), tokio::io::Error> {
-//!     let config = Default::default();
+//! use async_circe::{commands::Command, Client, Config};
+//!
+//! #[tokio::main(flavor = "current_thread")]
+//! async fn main() -> Result<(), tokio::io::Error> {
+//!     let config = Config::default();
 //!     let mut client = Client::new(config).await.unwrap();
 //!     client.identify().await.unwrap();
 //!
 //!     loop {
-//!         if let Ok(ref command) = client.read().await {
-//!             if let Command::OTHER(line) = command {
-//!                 print!("{}", line);
-//!             }
+//!         if let Some(ref command) = client.read().await? {
 //!             if let Command::PRIVMSG(nick, channel, message) = command {
-//!                println!("PRIVMSG received from {}: {} {}", nick, channel, message);
+//!                 println!("{} in {}: {}", nick, channel, message);
 //!             }
 //!         }
-//!         # break;
 //!     }
-//!     # Ok(())
 //! }
+//! ```
+//!
+//! The crate requires `tokio` with the `macros` and `rt` feature enabled to function.<br>
+//! For more examples (connecting to IRCs that dont use SSL, getting the config from
+//! a toml file, debugging or just in general an overview how a project with async-circe
+//! is structured) see the [examples](https://git.karx.xyz/circe/async-circe/src/branch/master/examples) folder on our git.
 
-#![warn(missing_docs)]
-#![allow(clippy::too_many_lines)]
+#![warn(missing_docs)] // We want everything documented
+#![allow(clippy::needless_return)] // Wants to remove a return statement, but when it's removed the code doesn't compile
 #![cfg_attr(docsrs, feature(doc_cfg))]
+#![feature(doc_cfg)]
 
-#[cfg(feature = "tls")]
-use async_native_tls::TlsConnector;
-
-use tokio::io::{AsyncReadExt, AsyncWriteExt, Error};
+use tokio::io::BufReader;
+use tokio::io::Error;
+use tokio::io::{AsyncBufReadExt, AsyncWriteExt};
 use tokio::net::TcpStream;
 
-#[cfg(feature = "toml_support")]
+#[cfg(feature = "tls")]
+use native_tls::TlsConnector;
+
+#[cfg(feature = "toml_config")]
 use serde_derive::Deserialize;
-#[cfg(feature = "toml_support")]
+#[cfg(feature = "toml_config")]
 use std::fs::File;
-#[cfg(feature = "toml_support")]
+#[cfg(feature = "toml_config")]
+use std::io::Read;
+#[cfg(any(doc, feature = "toml_config"))]
 use std::path::Path;
+#[cfg(feature = "toml_config")]
+use tokio::io::ErrorKind;
 
-/// IRC comamnds
 pub mod commands;
 
 /// An IRC client
 pub struct Client {
     config: Config,
     #[cfg(not(feature = "tls"))]
-    stream: TcpStream,
+    buf_reader: BufReader<TcpStream>,
     #[cfg(feature = "tls")]
-    stream: async_native_tls::TlsStream<TcpStream>,
+    buf_reader: BufReader<tokio_native_tls::TlsStream<tokio::net::TcpStream>>,
 }
 
-/// Config for the IRC client
+/// Config for the IRC client<br>
+/// For more information about what arguments the IRC commands take, see [`commands::Command`].
 #[derive(Clone, Debug, Default)]
-#[cfg_attr(feature = "toml_support", derive(Deserialize))]
+#[cfg_attr(feature = "toml_config", derive(Deserialize))]
 pub struct Config {
     channels: Vec<String>,
     host: String,
@@ -61,63 +71,60 @@ pub struct Config {
     username: String,
 }
 
-/// Custom Error for the `read` function
-#[derive(Debug)]
-pub struct NoNewLines;
-
-impl std::fmt::Display for NoNewLines {
-    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
-        write!(f, "Now new lines from the stream.")
-    }
-}
-
-impl std::error::Error for NoNewLines {}
-
 impl Client {
     /// Creates a new client with a given [`Config`].
+    /// # Example
     /// ```no_run
-    /// # use circe::*;
+    /// # #[tokio::main(flavor = "current_thread")]
+    /// # async fn main() -> Result<(), tokio::io::Error> {
     /// # let config = Default::default();
-    /// let mut client = Client::new(config).await.unwrap()?;
-    /// # Ok::<(), tokio::io::Error>(())
+    /// let mut client = Client::new(config).await?;
+    /// # Ok(())
+    /// # }
     /// ```
     /// # Errors
     /// Returns error if the client could not connect to the host.
     /// # Panics
     /// Panics if the client can't connect to the given host.
     pub async fn new(config: Config) -> Result<Self, Error> {
+        tracing::debug!("Creating TcpStream");
         let stream = TcpStream::connect(format!("{}:{}", config.host, config.port))
             .await
             .unwrap();
-
-        #[cfg(feature = "tls")]
-        let sslstream: async_native_tls::TlsStream<TcpStream>;
+        tracing::debug!("New TcpStream created");
 
         #[cfg(feature = "tls")]
         {
-            let connector = TlsConnector::new();
-            sslstream = connector
+            let connector = TlsConnector::builder().build().unwrap();
+            let async_connector = tokio_native_tls::TlsConnector::from(connector);
+
+            let sslstream = async_connector
                 .connect(config.host.as_str(), stream)
                 .await
                 .unwrap();
+
+            let buf_reader = BufReader::new(sslstream);
+
+            return Ok(Self { config, buf_reader });
         };
 
-        #[cfg(feature = "tls")]
-        return Ok(Self {
-            config,
-            stream: sslstream,
-        });
-
         #[cfg(not(feature = "tls"))]
-        return Ok(Self { config, stream });
+        {
+            let buf_reader = BufReader::new(stream);
+            return Ok(Self { config, buf_reader });
+        };
     }
 
     /// Identify user and joins the in the [`Config`] specified channels.
+    /// # Example
     /// ```no_run
-    /// # use circe::*;
-    /// # let mut client = Client::new(Default::default())?;
-    /// client.identify()?.await.unwrap();
-    /// # Ok::<(), tokio::io::Error>(())
+    /// # #[tokio::main(flavor = "current_thread")]
+    /// # async fn main() -> Result<(), tokio::io::Error> {
+    /// # let config = Default::default();
+    /// # let mut client = Client::new(config).await?;
+    /// client.identify().await?;
+    /// # Ok(())
+    /// # }
     /// ```
     /// # Errors
     /// Returns error if the client could not write to the stream.
@@ -140,13 +147,13 @@ impl Client {
         }
 
         loop {
-            if let Ok(ref command) = self.read().await {
+            if let Some(command) = self.read().await? {
                 match command {
                     commands::Command::PING(code) => {
-                        self.pong(code).await?;
+                        self.pong(&code).await?;
                     }
                     commands::Command::OTHER(line) => {
-                        tracing::debug!("{}", line);
+                        tracing::trace!("{}", line);
                         if line.contains("001") {
                             break;
                         }
@@ -173,205 +180,106 @@ impl Client {
             self.join(channel).await?;
         }
 
-        tracing::info!("circe has identified");
+        tracing::info!("async-circe has identified");
         Ok(())
     }
 
-    async fn read_string(&mut self) -> Option<String> {
-        let mut buffer = [0u8; 512];
+    async fn read_string(&mut self) -> Result<Option<String>, tokio::io::Error> {
+        let mut buffer = String::new();
 
-        let num_bytes = match self.stream.read(&mut buffer).await {
+        let num_bytes = match self.buf_reader.read_line(&mut buffer).await {
             Ok(b) => b,
-            Err(_) => 0,
+            Err(e) => {
+                tracing::error!("{}", e);
+                return Err(e);
+            }
         };
 
         if num_bytes == 0 {
-            return None;
+            return Ok(None);
         }
 
-        // Slice to the number of bytes that were actually read
-        let res = String::from_utf8_lossy(&buffer[..num_bytes]);
-
-        Some(res.into())
+        Ok(Some(buffer))
     }
 
     /// Read data coming from the IRC as a [`commands::Command`].
+    /// # Example
     /// ```no_run
-    /// # use circe::*;
-    /// # use circe::commands::Command;
-    /// # fn main() -> Result<(), tokio::io::Error> {
+    /// # #[tokio::main(flavor = "current_thread")]
+    /// # async fn main() -> Result<(), tokio::io::Error> {
     /// # let config = Default::default();
-    /// # let mut client = Client::new(config)?;
-    /// if let Ok(ref command) = client.read().await {
-    ///     if let Command::OTHER(line) = command {
-    ///         print!("{}", line);
-    ///     }
+    /// # let mut client = Client::new(config).await?;
+    /// # client.identify().await?;
+    /// loop {
+    ///    if let Some(ref command) = client.read().await? {
+    ///        if let commands::Command::PRIVMSG(nick, channel, message) = command {
+    ///            println!("{} in {}: {}", nick, channel, message);
+    ///        }
+    ///    }
+    /// # break;
     /// }
-    /// # Ok::<(), tokio::io::Error>(())
+    /// # Ok(())
     /// # }
     /// ```
     /// # Errors
-    /// Returns error if there are no new messages. This should not be taken as an actual error, because nothing went wrong.
-    pub async fn read(&mut self) -> Result<commands::Command, NoNewLines> {
-        if let Some(string) = self.read_string().await {
+    /// Returns IO errors from the TcpStream.
+    pub async fn read(&mut self) -> Result<Option<commands::Command>, tokio::io::Error> {
+        if let Some(string) = self.read_string().await? {
             let command = commands::Command::command_from_str(&string).await;
 
             if let commands::Command::PING(command) = command {
                 if let Err(_e) = self.pong(&command).await {
-                    return Err(NoNewLines);
+                    return Ok(None);
                 }
-                return Ok(commands::Command::PONG("".to_string()));
+                return Ok(Some(commands::Command::PONG("".to_string())));
             }
 
-            return Ok(command);
+            return Ok(Some(command));
         }
 
-        Err(NoNewLines)
+        Ok(None)
     }
 
     async fn write(&mut self, data: String) -> Result<(), Error> {
-        tracing::debug!("{:?}", data);
-        self.stream.write_all(data.as_bytes()).await?;
+        tracing::trace!("{:?}", data);
+        self.buf_reader.write_all(data.as_bytes()).await?;
 
         Ok(())
     }
 
-    /// Send a [`commands::Command`] to the IRC.<br>
-    /// Not reccomended to use, use the helper functions instead.
+    /// Request information about the admin of a given server.
+    /// # Example
     /// ```no_run
-    /// # use circe::*;
-    /// # use circe::commands::Command;
-    /// # let mut client = Client::new(Default::default())?;
-    /// client.write_command(Command::PRIVMSG("".to_string(), "#main".to_string(), "Hello".to_string())).await?;
-    /// # Ok::<(), tokio::io::Error>(())
+    /// # #[tokio::main(flavor = "current_thread")]
+    /// # async fn main() -> Result<(), tokio::io::Error> {
+    /// # let config = Default::default();
+    /// # let mut client = Client::new(config).await?;
+    /// # client.identify().await?;
+    /// client.admin("libera.chat").await?;
+    /// # Ok(())
+    /// # }
     /// ```
     /// # Errors
-    /// Returns error if the client could not write to the stream.
-    #[deprecated]
-    pub async fn write_command(&mut self, command: commands::Command) -> Result<(), Error> {
-        use commands::Command::{
-            ADMIN, AWAY, CAP, INVITE, JOIN, LIST, MODE, NAMES, NICK, OPER, OTHER, PART, PASS, PING,
-            PONG, PRIVMSG, QUIT, TOPIC, USER,
-        };
-        let computed = match command {
-            ADMIN(target) => {
-                format!("ADMIN {}\r\n", target)
-            }
-            AWAY(message) => {
-                format!("AWAY {}\r\n", message)
-            }
-            CAP(mode) => {
-                use commands::CapMode::{END, LS};
-                let formatted = {
-                    match mode {
-                        LS => "CAP LS 302\r\n",
-                        END => "CAP END\r\n",
-                    }
-                };
-                formatted.to_string()
-            }
-            INVITE(username, channel) => {
-                format!("INVITE {} {}\r\n", username, channel)
-            }
-            JOIN(channel) => {
-                format!("JOIN {}\r\n", channel)
-            }
-            LIST(channel, server) => {
-                let mut formatted = "LIST".to_string();
-                if let Some(channel) = channel {
-                    formatted.push_str(format!(" {}", channel).as_str());
-                }
-                if let Some(server) = server {
-                    formatted.push_str(format!(" {}", server).as_str());
-                }
-                formatted.push_str("\r\n");
-                formatted
-            }
-            NAMES(channel, server) => {
-                if let Some(server) = server {
-                    format!("NAMES {} {}\r\n", channel, server)
-                } else {
-                    format!("NAMES {}\r\n", channel)
-                }
-            }
-            NICK(nickname) => {
-                format!("NICK {}\r\n", nickname)
-            }
-            MODE(target, mode) => {
-                if let Some(mode) = mode {
-                    format!("MODE {} {}\r\n", target, mode)
-                } else {
-                    format!("MODE {}\r\n", target)
-                }
-            }
-            OPER(nick, password) => {
-                format!("OPER {} {}\r\n", nick, password)
-            }
-            OTHER(_) => {
-                return Err(Error::new(
-                    tokio::io::ErrorKind::Other,
-                    "Cannot write commands of type OTHER",
-                ));
-            }
-            PART(target) => {
-                format!("PART {}\r\n", target)
-            }
-            PASS(password) => {
-                format!("PASS {}\r\n", password)
-            }
-            PING(target) => {
-                format!("PING {}\r\n", target)
-            }
-            PONG(code) => {
-                format!("PONG {}\r\n", code)
-            }
-            PRIVMSG(_, target, message) => {
-                format!("PRIVMSG {} {}\r\n", target, message)
-            }
-            QUIT(message) => {
-                format!("QUIT :{}\r\n", message)
-            }
-            TOPIC(channel, topic) => {
-                if let Some(topic) = topic {
-                    format!("TOPIC {} :{}\r\n", channel, topic)
-                } else {
-                    format!("TOPIC {}\r\n", channel)
-                }
-            }
-            USER(username, s1, s2, realname) => {
-                format!("USER {} {} {} :{}\r\n", username, s1, s2, realname)
-            }
-        };
-
-        self.write(computed).await?;
-        Ok(())
-    }
-
-    // Helper commands
-
-    /// Helper function for requesting information about the ADMIN of an IRC server
-    /// ```no_run
-    /// # use circe::*;
-    /// # let mut client = Client::new(Default::default()).await?;
-    /// client.admin("192.168.178.100").await?;
-    /// # Ok::<(), tokio::io::Error>(())
-    /// ```
-    /// # Errors
-    /// Returns error if the client could not write to the stream.
+    /// Returns IO errors from the TcpStream.
     pub async fn admin(&mut self, target: &str) -> Result<(), Error> {
         self.write(format!("ADMIN {}\r\n", target)).await?;
         Ok(())
     }
 
-    /// Helper function for setting the users status to AWAY
+    /// Set the status of the client.
+    /// # Example
     /// ```no_run
-    /// # use circe::*;
-    /// # let mut client = Client::new(Default::default()).await?;
-    /// client.away("AFK").await?;
-    /// # Ok::<(), tokio::io::Error>(())
+    /// # #[tokio::main(flavor = "current_thread")]
+    /// # async fn main() -> Result<(), tokio::io::Error> {
+    /// # let config = Default::default();
+    /// # let mut client = Client::new(config).await?;
+    /// # client.identify().await?;
+    /// client.away("afk").await?;
+    /// # Ok(())
+    /// # }
     /// ```
     /// # Errors
-    /// Returns error if the client could not write to the stream.
+    /// Returns IO errors from the TcpStream.
     pub async fn away(&mut self, message: &str) -> Result<(), Error> {
         self.write(format!("AWAY {}\r\n", message)).await?;
         Ok(())
@@ -387,44 +295,59 @@ impl Client {
         Ok(())
     }
 
-    /// Helper function to INVITE people to a channels
+    /// Invite someone to a channel.
+    /// # Example
     /// ```no_run
-    /// # use circe::*;
-    /// # let mut client = Client::new(Default::default()).await?;
-    /// client.invite("liblemonirc", "#circe").await?;
-    /// # Ok::<(), tokio::io::Error>(())
+    /// # #[tokio::main(flavor = "current_thread")]
+    /// # async fn main() -> Result<(), tokio::io::Error> {
+    /// # let config = Default::default();
+    /// # let mut client = Client::new(config).await?;
+    /// # client.identify().await?;
+    /// client.invite("liblemonirc", "#async_circe").await?;
+    /// # Ok(())
+    /// # }
     /// ```
     /// # Errors
-    /// Returns error if the client could not write to the stream.
+    /// Returns IO errors from the TcpStream.
     pub async fn invite(&mut self, username: &str, channel: &str) -> Result<(), Error> {
         self.write(format!("INVITE {} {}\r\n", username, channel))
             .await?;
         Ok(())
     }
 
-    /// Helper function for sending JOINs.
+    /// Join a channel.
+    /// # Example
     /// ```no_run
-    /// # use circe::*;
-    /// # let mut client = Client::new(Default::default()).await?;
+    /// # #[tokio::main(flavor = "current_thread")]
+    /// # async fn main() -> Result<(), tokio::io::Error> {
+    /// # let config = Default::default();
+    /// # let mut client = Client::new(config).await?;
+    /// # client.identify().await?;
     /// client.join("#main").await?;
-    /// # Ok::<(), tokio::io::Error>(())
+    /// # Ok(())
+    /// # }
     /// ```
     /// # Errors
-    /// Returns error if the client could not write to the stream.
+    /// Returns IO errors from the TcpStream.
     pub async fn join(&mut self, channel: &str) -> Result<(), Error> {
         self.write(format!("JOIN {}\r\n", channel)).await?;
         Ok(())
     }
 
-    /// Helper function for ``LISTing`` channels and modes
+    /// List available channels on an IRC, or users in a channel.
+    /// # Example
     /// ```no_run
-    /// # use circe::*;
-    /// # let mut client = Client::new(Default::default()).await?;
+    /// # #[tokio::main(flavor = "current_thread")]
+    /// # async fn main() -> Result<(), tokio::io::Error> {
+    /// # let config = Default::default();
+    /// # let mut client = Client::new(config).await?;
+    /// # client.identify().await?;
     /// client.list(None, None).await?;
-    /// # Ok::<(), tokio::io::Error>(())
+    /// # Ok(())
+    /// # }
     /// ```
     /// # Errors
-    /// Returns error if the client could not write to the stream.
+    /// Returns IO errors from the TcpStream.
     pub async fn list(&mut self, channel: Option<&str>, server: Option<&str>) -> Result<(), Error> {
         let mut formatted = "LIST".to_string();
         if let Some(channel) = channel {
@@ -438,15 +361,20 @@ impl Client {
         Ok(())
     }
 
-    /// Helper function for sending MODEs.
+    /// Set the mode for a user.
+    /// # Example
     /// ```no_run
-    /// # use circe::*;
-    /// # let mut client = Client::new(Default::default()).await?;
+    /// # #[tokio::main(flavor = "current_thread")]
+    /// # async fn main() -> Result<(), tokio::io::Error> {
+    /// # let config = Default::default();
+    /// # let mut client = Client::new(config).await?;
+    /// # client.identify().await?;
     /// client.mode("test", Some("+B")).await?;
-    /// # Ok::<(), tokio::io::Error>(())
+    /// # Ok(())
+    /// # }
     /// ```
     /// # Errors
-    /// Returns error if the client could not write to the stream.
+    /// Returns IO errors from the TcpStream.
     pub async fn mode(&mut self, target: &str, mode: Option<&str>) -> Result<(), Error> {
         if let Some(mode) = mode {
             self.write(format!("MODE {} {}\r\n", target, mode,)).await?;
@@ -456,15 +384,20 @@ impl Client {
         Ok(())
     }
 
-    /// Helper function for getting all nicknames in a channel
+    /// Get all the people online in channels.
+    /// # Example
     /// ```no_run
-    /// # use circe::*;
-    /// # let mut client = Client::new(Default::default()).await?;
-    /// client.names("#main,#circe", None).await?;
-    /// # Ok::<(), tokio::io::Error>(())
+    /// # #[tokio::main(flavor = "current_thread")]
+    /// # async fn main() -> Result<(), tokio::io::Error> {
+    /// # let config = Default::default();
+    /// # let mut client = Client::new(config).await?;
+    /// # client.identify().await?;
+    /// client.names("#main,#async_circe", None).await?;
+    /// # Ok(())
+    /// # }
     /// ```
     /// # Errors
-    /// Returns error if the client could not write to the stream.
+    /// Returns IO errors from the TcpStream.
     pub async fn names(&mut self, channel: &str, server: Option<&str>) -> Result<(), Error> {
         if let Some(server) = server {
             self.write(format!("NAMES {} {}\r\n", channel, server))
@@ -475,44 +408,59 @@ impl Client {
         Ok(())
     }
 
-    /// Helper function for changeing your nick
+    /// Change your nickname on a server.
+    /// # Example
     /// ```no_run
-    /// # use circe::*;
-    /// # let mut client = Client::new(Default::default()).await?;
-    /// client.nick("Not circe").await?;
-    /// # Ok::<(), tokio::io::Error>(())
+    /// # #[tokio::main(flavor = "current_thread")]
+    /// # async fn main() -> Result<(), tokio::io::Error> {
+    /// # let config = Default::default();
+    /// # let mut client = Client::new(config).await?;
+    /// # client.identify().await?;
+    /// client.nick("Not async-circe").await?;
+    /// # Ok(())
+    /// # }
     /// ```
     /// # Errors
-    /// Returns error if the client could not write to the stream.
+    /// Returns IO errors from the TcpStream.
     pub async fn nick(&mut self, nickname: &str) -> Result<(), Error> {
         self.write(format!("NICK {}\r\n", nickname)).await?;
         Ok(())
     }
 
-    /// Helper function to try to register as a channel operator
+    /// Authentificate as an operator on a server.
+    /// # Example
     /// ```no_run
-    /// # use circe::*;
-    /// # let mut client = Client::new(Default::default()).await?;
+    /// # #[tokio::main(flavor = "current_thread")]
+    /// # async fn main() -> Result<(), tokio::io::Error> {
+    /// # let config = Default::default();
+    /// # let mut client = Client::new(config).await?;
+    /// # client.identify().await?;
     /// client.oper("username", "password").await?;
-    /// # Ok::<(), tokio::io::Error>(())
+    /// # Ok(())
+    /// # }
     /// ```
     /// # Errors
-    /// Returns error if the client could not write to the stream.
+    /// Returns IO errors from the TcpStream.
     pub async fn oper(&mut self, username: &str, password: &str) -> Result<(), Error> {
         self.write(format!("OPER {} {}\r\n", username, password))
             .await?;
         Ok(())
     }
 
-    /// Helper function for leaving channels.
+    /// Leave a channel.
+    /// # Example
     /// ```no_run
-    /// # use circe::*;
-    /// # let mut client = Client::new(Default::default()).await?;
+    /// # #[tokio::main(flavor = "current_thread")]
+    /// # async fn main() -> Result<(), tokio::io::Error> {
+    /// # let config = Default::default();
+    /// # let mut client = Client::new(config).await?;
+    /// # client.identify().await?;
     /// client.part("#main").await?;
-    /// # Ok::<(), tokio::io::Error>(())
+    /// # Ok(())
+    /// # }
     /// ```
     /// # Errors
-    /// Returns error if the client could not write to the stream.
+    /// Returns IO errors from the TcpStream.
     pub async fn part(&mut self, channel: &str) -> Result<(), Error> {
         self.write(format!("PART {}\r\n", channel)).await?;
         Ok(())
@@ -524,15 +472,20 @@ impl Client {
         Ok(())
     }
 
-    /// Tests the presence of a connection.
+    /// Tests the presence of a connection to a server.
+    /// # Example
     /// ```no_run
-    /// # use circe::*;
-    /// # let mut client = Client::new(Default::default()).await?;
+    /// # #[tokio::main(flavor = "current_thread")]
+    /// # async fn main() -> Result<(), tokio::io::Error> {
+    /// # let config = Default::default();
+    /// # let mut client = Client::new(config).await?;
+    /// # client.identify().await?;
     /// client.ping("libera.chat", None).await?;
-    /// # Ok::<(), tokio::io::Error>(())
+    /// # Ok(())
+    /// # }
     /// ```
     /// # Errors
-    /// Returns error if the client could not write to the stream.
+    /// Returns IO errors from the TcpStream.
     pub async fn ping(&mut self, server1: &str, server2: Option<&str>) -> Result<(), Error> {
         if let Some(server2) = server2 {
             self.write(format!("PING {} {}\r\n", server1, server2))
@@ -550,54 +503,81 @@ impl Client {
         Ok(())
     }
 
-    /// Helper function for sending PRIVMSGs.
+    /// Send a message to a channel.
+    /// # Example
     /// ```no_run
-    /// # use circe::*;
-    /// # let mut client = Client::new(Default::default()).await?;
+    /// # #[tokio::main(flavor = "current_thread")]
+    /// # async fn main() -> Result<(), tokio::io::Error> {
+    /// # let config = Default::default();
+    /// # let mut client = Client::new(config).await?;
+    /// # client.identify().await?;
     /// client.privmsg("#main", "Hello").await?;
-    /// # Ok::<(), tokio::io::Error>(())
+    /// # Ok(())
+    /// # }
     /// ```
     /// # Errors
-    /// Returns error if the client could not write to the stream.
+    /// Returns IO errors from the TcpStream.
     pub async fn privmsg(&mut self, channel: &str, message: &str) -> Result<(), Error> {
         self.write(format!("PRIVMSG {} {}\r\n", channel, message))
             .await?;
         Ok(())
     }
 
-    /// Helper function for leaving the IRC server and shutting down the TCP stream afterwards.
+    /// Leave the IRC server you are connected to.
+    /// # Example
     /// ```no_run
-    /// # use circe::*;
-    /// # let mut client = Client::new(Default::default()).await?;
+    /// # #[tokio::main(flavor = "current_thread")]
+    /// # async fn main() -> Result<(), tokio::io::Error> {
+    /// # let config = Default::default();
+    /// # let mut client = Client::new(config).await?;
+    /// # client.identify().await?;
     /// client.quit(None).await?;
-    /// # Ok::<(), tokio::io::Error>(())
+    /// # Ok(())
+    /// # }
     /// ```
     /// # Errors
-    /// Returns error if the client could not write to the stream.
+    /// Returns IO errors from the TcpStream.
     pub async fn quit(&mut self, message: Option<&str>) -> Result<(), Error> {
         if let Some(message) = message {
             self.write(format!("QUIT :{}\r\n", message)).await?;
         } else {
             self.write(format!(
-                "QUIT :circe {} (https://crates.io/crates/async-circe)\r\n",
+                "QUIT :async-circe {} (https://crates.io/crates/async-circe)\r\n",
                 env!("CARGO_PKG_VERSION")
             ))
             .await?;
         }
-        tracing::info!("circe shutting down");
-        self.stream.shutdown().await?;
+        tracing::info!("async-circe shutting down");
+        self.buf_reader.shutdown().await?;
         Ok(())
     }
 
-    /// Helper function for setting or getting the topic of a channel
+    /// Get the topic of a channel.
+    /// # Example
     /// ```no_run
-    /// # use circe::*;
-    /// # let mut client = Client::new(Default::default()).await?;
+    /// # #[tokio::main(flavor = "current_thread")]
+    /// # async fn main() -> Result<(), tokio::io::Error> {
+    /// # let config = Default::default();
+    /// # let mut client = Client::new(config).await?;
+    /// # client.identify().await?;
+    /// client.topic("#main", None).await?;
+    /// # Ok(())
+    /// # }
+    /// ```
+    /// Set the topic of a channel.
+    /// # Example
+    /// ```no_run
+    /// # #[tokio::main(flavor = "current_thread")]
+    /// # async fn main() -> Result<(), tokio::io::Error> {
+    /// # let config = Default::default();
+    /// # let mut client = Client::new(config).await?;
+    /// # client.identify().await?;
     /// client.topic("#main", Some("main channel")).await?;
-    /// # Ok::<(), tokio::io::Error>(())
+    /// # Ok(())
+    /// # }
     /// ```
     /// # Errors
-    /// Returns error if the client could not write to the stream.
+    /// Returns IO errors from the TcpStream.
     pub async fn topic(&mut self, channel: &str, topic: Option<&str>) -> Result<(), Error> {
         if let Some(topic) = topic {
             self.write(format!("TOPIC {} :{}\r\n", channel, topic))
@@ -624,18 +604,18 @@ impl Client {
 
 impl Config {
     /// Create a new config for the client
-    ///
+    /// # Example
     /// ```rust
-    /// # use circe::*;
     /// let config = Config::new(
-    ///     &["#main", "#circe"],
+    ///     &["#main", "#async_circe"],
     ///     "karx.xyz",
     ///     Some("+B"),
-    ///     Some("circe"),
+    ///     Some("async-circe"),
     ///     6697,
     ///     "circe",
     /// );
     /// ```
+    #[must_use]
     pub fn new(
         channels: &[&'static str],
         host: &str,
@@ -644,7 +624,7 @@ impl Config {
         port: u16,
         username: &str,
     ) -> Self {
-        tracing::info!("New circe client config");
+        tracing::info!("New async-circe client config");
         // Conversion from &'static str to String
         let channels_config = channels
             .iter()
@@ -674,11 +654,23 @@ impl Config {
             username: username.into(),
         };
 
-        tracing::debug!("circe config: {:?}", config);
+        tracing::debug!("async-circe config: {:?}", config);
         config
     }
 
-    /// Allows for configuring circe at runtime
+    /// Allows for configuring async-circe at runtime.
+    /// # Example
+    /// ```no_run
+    /// let config = Config::runtime_config(
+    ///     vev!["#main".to_string(), "#async_circe".to_string()],
+    ///     "karx.xyz".to_string(),
+    ///     Some("+B".to_string()),
+    ///     Some("async-circe".to_string()),
+    ///     6697,
+    ///     "circe".to_string(),
+    /// );
+    /// ```
+    #[must_use]
     pub fn runtime_config(
         channels: Vec<String>,
         host: String,
@@ -687,17 +679,17 @@ impl Config {
         port: u16,
         username: String,
     ) -> Self {
-        tracing::info!("New circe client config");
+        tracing::info!("New async-circe client config");
         let mode_config: Option<String>;
         if let Some(mode) = mode {
-            mode_config = Some(mode.to_string());
+            mode_config = Some(mode);
         } else {
             mode_config = None;
         }
 
         let nickname_config: Option<String>;
         if let Some(nickname) = nickname {
-            nickname_config = Some(nickname.to_string());
+            nickname_config = Some(nickname);
         } else {
             nickname_config = Some(username.to_string());
         }
@@ -711,36 +703,37 @@ impl Config {
             username,
         };
 
-        tracing::debug!("circe config: {:?}", config);
+        tracing::debug!("async-circe config: {:?}", config);
         config
     }
 
     /// Create a config from a toml file
+    /// # Example
     /// ```toml
-    /// channels = ["#main", "#main2"]
-    /// host = "192.168.178.100"
+    /// #Config.toml
+    /// channels = ["#main", "#circe"]
+    /// host = "karx.xyz"
     /// mode = "+B"
-    /// nickname = "circe"
+    /// nickname = "async-circe"
     /// port = 6667
     /// username = "circe"
     /// ```
+    /// ```no_run
+    /// let config = Config::from_toml("Config.toml")?;
+    /// ```
     /// # Errors
     /// Returns an Error if the file cannot be opened or if the TOML is invalid
-    ///
-    #[cfg_attr(docsrs, doc(cfg(feature = "toml_support")))]
-    #[cfg(feature = "toml_support")]
+    #[cfg(any(doc, feature = "toml_config"))]
+    #[doc(cfg(feature = "toml_config"))]
     pub fn from_toml<P: AsRef<Path>>(path: P) -> Result<Self, Error> {
-        tracing::info!("New circe client config");
-        use std::io::Read;
+        tracing::info!("New async-circe client config");
         let mut file = File::open(&path)?;
         let mut data = String::new();
         file.read_to_string(&mut data)?;
 
-        let config = toml::from_str(&data).map_err(|e| {
-            use tokio::io::ErrorKind;
-            Error::new(ErrorKind::Other, format!("Invalid TOML: {}", e))
-        });
-        tracing::debug!("circe config: {:?}", config);
+        let config = toml::from_str(&data)
+            .map_err(|e| Error::new(ErrorKind::Other, format!("Invalid TOML: {}", e)));
+        tracing::debug!("async-circe config: {:?}", config);
         config
     }
 }
